"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var crypto = require("crypto");
var hmac = function (key, string) {
    return crypto.createHmac("sha256", key).update(string, "utf8").digest();
};
var hash = function (string) {
    return crypto.createHash("sha256")
        .update(string, "utf8")
        .digest("hex");
};
var trimAll = function (string) {
    return string.trim().replace(/\s+/g, " ");
};
var encodeRfc3986 = function (urlEncodedString) {
    return urlEncodedString.replace(/[!'()*]/g, function (c) {
        return '%' + c.charCodeAt(0).toString(16).toUpperCase();
    });
};
var createCanonicalRequest = function (httpRequestMethod, canonicalURIRaw, queryParams, headers, requestPayload) {
    if (requestPayload === void 0) { requestPayload = ""; }
    var canonicalURI;
    if (canonicalURIRaw === "/") {
        canonicalURI = canonicalURIRaw;
    }
    else {
        canonicalURI = encodeRfc3986(encodeURIComponent(canonicalURIRaw));
    }
    var canonicalQueryString = queryParams.map(function (param) {
        return param.map(encodeURIComponent).join("=");
    }).join("&");
    var signedHeaderStrings = [];
    var canonicalHeaders = headers.map(function (header) {
        header[0] = header[0].toLowerCase();
        signedHeaderStrings.push(header[0]);
        header[1] = trimAll(header[1]);
        return header.join(":") + "\n";
    }).join("");
    var signedHeaders = signedHeaderStrings.join(";");
    return (httpRequestMethod + "\n" +
        (canonicalURI + "\n") +
        (canonicalQueryString + "\n") +
        (canonicalHeaders + "\n") +
        (signedHeaders + "\n") +
        ("" + hash(requestPayload)));
};
var createStringToSign = function (awsDateTime, region, service, canonicalRequest) {
    return ("AWS4-HMAC-SHA256\n" +
        (awsDateTime.getLong() + "\n") +
        (awsDateTime.getShort() + "/" + region + "/" + service + "/aws4_request\n") +
        ("" + hash(canonicalRequest)));
};
var calculateSignature = function (secretAccessKey, awsDateTime, region, service, stringToSign) {
    var kDate = hmac("AWS4" + secretAccessKey, awsDateTime.getShort());
    var kRegion = hmac(kDate, region);
    var kSerivce = hmac(kRegion, service);
    var kSigning = hmac(kSerivce, "aws4_request");
    return crypto.createHmac("sha256", kSigning)
        .update(stringToSign, "utf8")
        .digest("hex");
};
var aws4 = {
    sign: function (httpRequestMethod, service, region, path, queryParams, awsDateTime, headers, payload, secretAccessKey) {
        var canonicalRequest = createCanonicalRequest(httpRequestMethod, path, queryParams, headers, payload);
        var stringToSign = createStringToSign(awsDateTime, region, service, canonicalRequest);
        return calculateSignature(secretAccessKey, awsDateTime, region, service, stringToSign);
    },
};
exports.default = aws4;
